==> ben-2.08.tar.gz: ben/queue.c <==
/*
    This file is part of ben -- a task scheduler for distributed computations
    Copyright (C) 2012-2019  Laurent Poirrier

    ben is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include "xmem.h"
#include "exec.h"
#include "queue.h"


// ***************************************************************************
// 
// ***************************************************************************
job *job_new(int id,
	char *shell, char *dir, char *options, char *name, char *cmd)
{
	job *t;
	
	t = xmalloc(sizeof(job));
	
	t->id = id;
	t->shell = xstrdup(shell);
	t->dir = xstrdup(dir);
	t->options = xstrdup(options);
	t->name = xstrdup(name);
	t->cmd = xstrdup(cmd);
	t->restrict_box = -1;
	t->forward_box = -1;
	t->ran_box = -1;
	t->outpath = xprintf(NULL, "%s%s%s.out",
		dir, ((dir[0]) ? "/" : ""), name);
	t->errpath = xprintf(NULL, "%s%s%s.log",
		dir, ((dir[0]) ? "/" : ""), name);
	t->outfd = -1;
	t->errfd = -1;
	t->total_out = 0;
	t->total_err = 0;
	t->start_time = 0;
	t->stop_time = 0;
	
	return(t);
}

// ***************************************************************************
// 
// ***************************************************************************
void job_free(job *t)
{
	xfree(t->shell);
	xfree(t->dir);
	xfree(t->options);
	xfree(t->name);
	xfree(t->cmd);
	xfree(t->outpath);
	xfree(t->errpath);
	xfree(t);
}

// ***************************************************************************
// 
// ***************************************************************************
int job_open(job *t)
{
	t->total_out = 0;
	t->total_err = 0;
	
	t->start_time = time(NULL);
	t->stop_time = 0;
	
	if (t->forward_box >= 0)
		return(0);
	
	if (t->dir[0]) {
		exec_recursive_mkdir(t->dir);
	}
	
	exec_path_mkdir(t->outpath);
	t->outfd = creat(t->outpath, 0600);
	if (t->outfd < 0) {
		fprintf(stderr, "%s: %s (creating file)\n",
			t->outpath, strerror(errno));
		job_close(t, 1);
		return(-1);
	}
	
	exec_path_mkdir(t->errpath);
	t->errfd = creat(t->errpath, 0600);
	if (t->errfd < 0) {
		fprintf(stderr, "%s: %s (creating file)\n",
			t->errpath, strerror(errno));
		job_close(t, 1);
		return(-1);
	}
	
	return(0);
}

// ***************************************************************************
// 
// ***************************************************************************
void job_close(job *t, int rmfiles)
{
	if (t->outfd >= 0)
		close(t->outfd);
	if (t->errfd >= 0)
		close(t->errfd);
	
	if ((rmfiles) && (t->outfd >= 0)) {
		if (unlink(t->outpath)) {
			fprintf(stderr, "%s: %s (removing file)\n",
				t->outpath, strerror(errno));
		}
	}
	
	if ((rmfiles) && (t->errfd >= 0)) {
		if (unlink(t->errpath)) {
			fprintf(stderr, "%s: %s (removing file)\n",
				t->errpath, strerror(errno));
		}
	}
	
	t->outfd = -1;
	t->errfd = -1;
	t->total_out = 0;
	t->total_err = 0;
	
	t->stop_time = time(NULL);
}

// ***************************************************************************
// 
// ***************************************************************************
int job_matches(job *t, int id, char *dir, char *name)
{
	// id mismatch
	if ((id >= 0) && (t->id != id))
		return(0);
	
	// dir mismatch
	if ((dir) && (dir[0]) && (strcmp(dir, t->dir)))
		return(0);
	
	// name mismatch
	if ((name) && (name[0]) && (strcmp(name, t->name)))
		return(0);
	
	// match
	return(1);
}


// ***************************************************************************
// 
// ***************************************************************************
void queue_init(queue *q)
{
	q->head = NULL;
	q->tail = NULL;
	q->size = 0;
}

void queue_clear(queue *q)
{
	job *t, *u;
	
	t = q->head;
	
	while (t) {
		u = t->next;
		
		job_close(t, 0);
		job_free(t);
		
		t = u;
	}
	
	q->head = NULL;
	q->tail = NULL;
	q->size = 0;
}

// ***************************************************************************
// 
// ***************************************************************************
void queue_prepend(queue *q, job *t)
{
	t->prev = NULL;
	t->next = q->head;
	
	if (q->head)
		q->head->prev = t;
	
	q->head = t;
	
	if (!q->tail)
		q->tail = t;
	
	q->size++;
}

void queue_append(queue *q, job *t)
{
	t->prev = q->tail;
	t->next = NULL;
	
	if (q->tail)
		q->tail->next = t;
	
	q->tail = t;
	
	if (!q->head)
		q->head = t;
	
	q->size++;
}

void queue_unlink(queue *q, job *t)
{
	if (t->prev)
		t->prev->next = t->next;
	
	if (t->next)
		t->next->prev = t->prev;

	if (q->head == t)
		q->head = t->next;
	
	if (q->tail == t)
		q->tail = t->prev;
	
	t->prev = NULL;
	t->next = NULL;
	
	q->size--;
}

// ***************************************************************************
// 
// ***************************************************************************
job *queue_find(queue *q, int id)
{
	job *t;
	
	t = q->head;
	
	while (t) {
		if (t->id == id)
			return(t);
		
		t = t->next;
	}
	
	return(NULL);
}

==> ben-2.16.tar.gz: ben/queue.c <==
/*
    This file is part of ben -- a task scheduler for distributed computations
    Copyright (C) 2012-2023  Laurent Poirrier

    ben is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <limits.h>
#include <time.h>
#include "xmem.h"
#include "exec.h"
#include "queue.h"


// ***************************************************************************
// 
// ***************************************************************************
job *job_new(int id,
	char *shell, char *dir, char *options, char *name, char *cmd)
{
	job *t;
	
	t = xmalloc(sizeof(job));
	
	t->id = id;
	t->shell = xstrdup(shell);
	t->dir = xstrdup(dir);
	t->options = xstrdup(options);
	t->name = xstrdup(name);
	t->cmd = xstrdup(cmd);
	t->restrict_box = -1;
	t->forward_box = -1;
	t->ran_box = -1;
	t->outpath = xprintf(NULL, "%s%s%s.out",
		dir, ((dir[0]) ? "/" : ""), name);
	t->errpath = xprintf(NULL, "%s%s%s.log",
		dir, ((dir[0]) ? "/" : ""), name);
	t->outfd = -1;
	t->errfd = -1;
	t->total_out = 0;
	t->total_err = 0;
	t->start_time = 0;
	t->stop_time = 0;
	
	return(t);
}

// ***************************************************************************
// 
// ***************************************************************************
void job_free(job *t)
{
	xfree(t->shell);
	xfree(t->dir);
	xfree(t->options);
	xfree(t->name);
	xfree(t->cmd);
	xfree(t->outpath);
	xfree(t->errpath);
	xfree(t);
}

// ***************************************************************************
// 
// ***************************************************************************
int job_open(job *t)
{
	t->total_out = 0;
	t->total_err = 0;
	
	t->start_time = time(NULL);
	t->stop_time = 0;
	
	if (t->forward_box >= 0)
		return(0);
	
	if (t->dir[0]) {
		exec_recursive_mkdir(t->dir);
	}
	
	exec_path_mkdir(t->outpath);
	t->outfd = creat(t->outpath, 0600);
	if (t->outfd < 0) {
		fprintf(stderr, "%s: %s (creating file)\n",
			t->outpath, strerror(errno));
		job_close(t, 1);
		return(-1);
	}
	
	exec_path_mkdir(t->errpath);
	t->errfd = creat(t->errpath, 0600);
	if (t->errfd < 0) {
		fprintf(stderr, "%s: %s (creating file)\n",
			t->errpath, strerror(errno));
		job_close(t, 1);
		return(-1);
	}
	
	return(0);
}

// ***************************************************************************
// 
// ***************************************************************************
void job_close(job *t, int rmfiles)
{
	if (t->outfd >= 0)
		close(t->outfd);
	if (t->errfd >= 0)
		close(t->errfd);
	
	if ((rmfiles) && (t->outfd >= 0)) {
		if (unlink(t->outpath)) {
			fprintf(stderr, "%s: %s (removing file)\n",
				t->outpath, strerror(errno));
		}
	}
	
	if ((rmfiles) && (t->errfd >= 0)) {
		if (unlink(t->errpath)) {
			fprintf(stderr, "%s: %s (removing file)\n",
				t->errpath, strerror(errno));
		}
	}
	
	t->outfd = -1;
	t->errfd = -1;
	t->total_out = 0;
	t->total_err = 0;
	
	t->stop_time = time(NULL);
}

// ***************************************************************************
// 
// ***************************************************************************
int job_matches(job *t, pattern *dir, pattern *job)
{
	return pattern_matches(dir, 0, t->dir)
		&& pattern_matches(job, t->id, t->name);
}

// ***************************************************************************
// 
// ***************************************************************************
void queue_init(queue *q)
{
	q->head = NULL;
	q->tail = NULL;
	q->size = 0;
}

void queue_clear(queue *q)
{
	job *t, *u;
	
	t = q->head;
	
	while (t) {
		u = t->next;
		
		job_close(t, 0);
		job_free(t);
		
		t = u;
	}
	
	q->head = NULL;
	q->tail = NULL;
	q->size = 0;
}

// ***************************************************************************
// 
// ***************************************************************************
void queue_prepend(queue *q, job *t)
{
	t->prev = NULL;
	t->next = q->head;
	
	if (q->head)
		q->head->prev = t;
	
	q->head = t;
	
	if (!q->tail)
		q->tail = t;
	
	q->size++;
}

void queue_append(queue *q, job *t)
{
	t->prev = q->tail;
	t->next = NULL;
	
	if (q->tail)
		q->tail->next = t;
	
	q->tail = t;
	
	if (!q->head)
		q->head = t;
	
	q->size++;
}

void queue_unlink(queue *q, job *t)
{
	if (t->prev)
		t->prev->next = t->next;
	
	if (t->next)
		t->next->prev = t->prev;

	if (q->head == t)
		q->head = t->next;
	
	if (q->tail == t)
		q->tail = t->prev;
	
	t->prev = NULL;
	t->next = NULL;
	
	q->size--;
}

// ***************************************************************************
// 
// ***************************************************************************
job *queue_find(queue *q, int id)
{
	job *t;
	
	t = q->head;
	
	while (t) {
		if (t->id == id)
			return(t);
		
		t = t->next;
	}
	
	return(NULL);
}
